// bmp2png.c
// A sample program that uses the PNGDIB library
// converts a BMP image file to a PNG image file

#include <windows.h>
#include <stdio.h>
#include "pngdib.h"

//#define DIB_ALPHA32

// This is just a generic function to read a file into a memory block.
int read_bmp_to_mem(char *bmpfn,unsigned char **bmppp, DWORD *fsizep)
{
	HANDLE hfile;
	DWORD fsize;
	unsigned char *fbuf;
	DWORD bytesread;

	hfile=CreateFile(bmpfn,GENERIC_READ,FILE_SHARE_READ,NULL,
		OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	if(hfile==INVALID_HANDLE_VALUE) return 1;

	fsize=GetFileSize(hfile,NULL);
	if(fsize>0) {
		fbuf=(unsigned char*)GlobalAlloc(GPTR,fsize);
		if(fbuf) {
			if(ReadFile(hfile,(void*)fbuf,fsize,&bytesread,NULL)) {
				if(bytesread==fsize) { 
					(*bmppp)  = fbuf;
					(*fsizep) = fsize;
					CloseHandle(hfile);
					return 0;   // success
				}
			}
			GlobalFree((void*)fbuf);
		}
	}
	CloseHandle(hfile);
	return 1;  // error
}


int main(int argc,char **argv)
{
	char *bmpfn,*pngfn;
	PNGD_D2PINFO d2p;
	int ret;
	unsigned char *bmpp;
	DWORD bmpsize;
	LPBITMAPFILEHEADER lpbmfh;
	char errmsg[100];

	if(argc!=3) {
		printf("Usage: %s <file.bmp> <file.png>\n",argv[0]);
		return 1;
	}
	bmpfn=argv[1];
	pngfn=argv[2];


	if(read_bmp_to_mem(bmpfn,&bmpp, &bmpsize)) {
		printf("can't read BMP from file\n");
		return 1;
	}

	ZeroMemory((void*)&d2p,sizeof(PNGD_D2PINFO));

	d2p.structsize=sizeof(PNGD_D2PINFO);

	d2p.flags=0;
#ifdef DIB_ALPHA32
	d2p.flags |= PNGD_DIB_ALPHA32;
#endif

	d2p.pngfn=pngfn;

	// The first 14 bytes of a BMP file is a file header, the
	// rest is a DIB.
	d2p.lpdib = (LPBITMAPINFOHEADER)&bmpp[14];

	// This can be left at 0 if you know your DIB is valid.
	d2p.dibsize = bmpsize-14;

	lpbmfh= (LPBITMAPFILEHEADER)bmpp;
	if(lpbmfh->bfOffBits >= bmpsize) {
		printf("Corrupt BMP\n");
		return 1;
	}

	// It's usually okay to leave lpbits set to NULL.
	d2p.lpbits = &bmpp[lpbmfh->bfOffBits];

	// This can be left at 0 if you know your DIB is valid.
	d2p.bitssize = bmpsize-lpbmfh->bfOffBits;

	d2p.software = "PNGDIB test app";
	d2p.errmsg = errmsg; strcpy(errmsg,"");

	ret=write_dib_to_png(&d2p);
	// returns 0 on success
	if(ret) {
		printf("Error: %s (%d)\n",d2p.errmsg,ret);
		return 1;
	}

	// 
	GlobalFree(bmpp);

	return 0;
}

