// png2dib.c
// sample using PNGDIB library
// convert a PNG image file to a BMP image file

#include <windows.h>
#include <stdio.h>
#include "pngdib.h"

//#define DIB_ALPHA32

int write_dib_to_bmp(char *bmpfn, LPBITMAPINFOHEADER lpdib, 
					 int dibsize, int bitsoffset)
{
	HANDLE hfile;
	BITMAPFILEHEADER h;
	DWORD written, err;

	ZeroMemory((void*)&h,sizeof(h));
	h.bfType= MAKEWORD('B','M');
	h.bfSize=    sizeof(BITMAPFILEHEADER)+dibsize;
	h.bfOffBits= sizeof(BITMAPFILEHEADER)+bitsoffset;

	hfile=CreateFile(bmpfn,GENERIC_WRITE,FILE_SHARE_READ,NULL,
		CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
	if(hfile==INVALID_HANDLE_VALUE) return 1;

	if(!WriteFile(hfile,(void*)&h,sizeof(BITMAPFILEHEADER),&written,NULL)) {
		err=GetLastError();
		CloseHandle(hfile);
		return 1;
	}
	if(!WriteFile(hfile,(void*)lpdib,dibsize,&written,NULL)) {
		err=GetLastError();
		CloseHandle(hfile);
		return 1;
	}
	CloseHandle(hfile);
	return 0;
}


int main(int argc,char **argv)
{
	char *bmpfn,*pngfn;
	PNGD_P2DINFO p2d;
	int ret;
	char errmsg[100];


	if(argc!=3) {
		printf("Usage: %s <file.png> <file.bmp>\n",argv[0]);
		return 1;
	}
	pngfn=argv[1];
	bmpfn=argv[2];


	ZeroMemory((void*)&p2d,sizeof(PNGD_P2DINFO));

	p2d.structsize=sizeof(PNGD_P2DINFO);
	p2d.flags=0;
#ifdef DIB_ALPHA32
	p2d.flags |= PNGD_DIB_ALPHA32;
#else 
	p2d.flags |= PNGD_GAMMA_CORRECTION;
#endif
	p2d.pngfn=pngfn;      // name of the file to read
	p2d.errmsg=errmsg; strcpy(errmsg,"");

	ret=read_png_to_dib(&p2d);
	// returns 0 on success
	if(ret) {
		// see pngdib.h for a list of error codes
		printf("Error: %s (%d)\n",errmsg,ret);
		return 1;
	}

	if(write_dib_to_bmp(bmpfn, p2d.lpdib, p2d.dibsize, p2d.bits_offs)) {
		printf("Can't write BMP file\n");
	}

	// The DIB will have been allocated with GlobalAlloc(GMEM_FIXED...).
	// You should free it with GlobalFree when you're done with it.
	GlobalFree((void*)p2d.lpdib);

	return 0;
}
