/* Image Viewer Demo
 * A really basic PNG image viewer for windows.
 * This program is a demonstration of the pngdib library.
 * It also requires the libpng and zlib libraries (needed by pngdib).
 *
 * By Jason Summers
 *
 * Copyright is hereby disclaimed on this source code.
 */

#include <windows.h>
#include <commdlg.h>
#include <stdio.h>
#include "resource.h"
#include "pngdib.h"

#define MYNAME "Simple PNG Image Viewer"
#define SMVIEW_VERS  "2.2.0"

HINSTANCE hInst;
LPBITMAPINFOHEADER lpdib;
int lpdib_size, lpdib_bits_offs;
void *lpbits;
int use_bkgd, use_custom_bg;
int use_gamma;
COLORREF custom_bg, default_bg, image_bg;
int which_bg;   /* 0==custom or default (depending on use_custom_bg) */
                /* 1==image's */
char png_filename[MAX_PATH];

static COLORREF g_custcolors[16];

LRESULT CALLBACK WndProcMain(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam);
BOOL CALLBACK DlgProcAbout(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam);

int choose_color_dialog(HWND hwnd, COLORREF *color)
{
	CHOOSECOLOR cc;

	ZeroMemory(&cc,sizeof(CHOOSECOLOR));
	cc.lStructSize=sizeof(CHOOSECOLOR);
	cc.hwndOwner=hwnd;
	cc.rgbResult= *color;  /* set initial value */
	cc.lpCustColors=g_custcolors;
	cc.Flags=CC_RGBINIT|CC_FULLOPEN;

	if(ChooseColor(&cc)) {
		(*color) = cc.rgbResult;
		return 1;
	}
	return 0;
}

int open_png_by_name(HWND hwnd, char *fn)
{
	PNGD_P2DINFO p2d;
	char buf[1000];
	int ret;
	char errmsg[100];

	if(!strlen(fn)) {
		InvalidateRect(hwnd,NULL,TRUE);
		return 0;
	}

	ZeroMemory((void*)&p2d,sizeof(PNGD_P2DINFO));
	p2d.structsize=sizeof(PNGD_P2DINFO);
	p2d.pngfn=fn;
	p2d.flags |=  (use_bkgd?PNGD_USE_BKGD:0)
		         | (use_custom_bg?PNGD_USE_CUSTOM_BG:0)
				 | (use_gamma?PNGD_GAMMA_CORRECTION:0)
				 | PNGD_USE_HEAPALLOC;
	p2d.bgcolor.red   = GetRValue(custom_bg);
	p2d.bgcolor.green = GetGValue(custom_bg);
	p2d.bgcolor.blue  = GetBValue(custom_bg);
	p2d.errmsg=errmsg; strcpy(errmsg,"");
	p2d.heap = GetProcessHeap();

	ret=read_png_to_dib(&p2d);
	if(ret) {    /* returns 0 on success */
		SetForegroundWindow(hwnd);
		sprintf(buf,"Cannot load image: %s (%d)",errmsg,ret);
		MessageBox(hwnd,buf,MYNAME,MB_OK|MB_ICONWARNING);
		InvalidateRect(hwnd,NULL,TRUE);
		return 0;
	}

	/* get rid of previous image */
	if(lpdib) { HeapFree(GetProcessHeap(),0,(LPVOID)lpdib); lpdib=NULL; }

	lpdib            = p2d.lpdib;
	lpdib_size       = p2d.dibsize;
	lpdib_bits_offs  = p2d.bits_offs;
	lpbits           = p2d.lpbits;

	InvalidateRect(hwnd,NULL,TRUE);
	strcpy(png_filename,fn);

	if(p2d.flags & PNGD_BG_RETURNED) {
		image_bg = RGB(p2d.bgcolor.red,p2d.bgcolor.green,p2d.bgcolor.blue);
		which_bg=1;
	}
	else {
		which_bg=0;
	}

	return 1;
}

int open_png_dialog(HWND hwnd)
{
	char fn[MAX_PATH];
	OPENFILENAME ofn;

	strcpy(fn,"");

	ZeroMemory(&ofn,sizeof(OPENFILENAME));

	ofn.lStructSize=sizeof(OPENFILENAME);
	ofn.hwndOwner=hwnd;
	ofn.lpstrFilter="PNG image (*.png)\0*.png\0All files\0*.*\0\0";
	ofn.nFilterIndex=1;
	ofn.lpstrTitle="Open PNG file";
	
	ofn.lpstrInitialDir=NULL;
	ofn.lpstrFile=fn;
	ofn.nMaxFile=MAX_PATH;
	ofn.Flags=OFN_FILEMUSTEXIST|OFN_HIDEREADONLY;

	if(!GetOpenFileName(&ofn)) return 0;  /* canceled by user */
	return open_png_by_name(hwnd,fn);
}

void dropped_files(HWND hwnd, HDROP hDrop)
{
	UINT ret;
	char fn[MAX_PATH];

	ret=DragQueryFile(hDrop,0xFFFFFFFF,NULL,0);
	if(ret==1) {  	/* number of files dropped */
		ret=DragQueryFile(hDrop,0,fn,MAX_PATH);
		if(ret) open_png_by_name(hwnd,fn);
	}
	DragFinish(hDrop);
}

void save_as_png(HWND hwnd)
{
	PNGD_D2PINFO d2p;
	int ret;
	OPENFILENAME ofn;
	char fn[MAX_PATH];
	char errmsg[100];
	char buf[200];

	strcpy(fn,png_filename);

	ZeroMemory(&ofn,sizeof(OPENFILENAME));
	ofn.lStructSize=sizeof(OPENFILENAME);
	ofn.hwndOwner=hwnd;
	ofn.lpstrFilter="PNG image (*.png)\0*.png\0\0";
	ofn.nFilterIndex=1;
	ofn.lpstrTitle="Save As PNG";
	
	ofn.lpstrInitialDir=NULL;
	ofn.lpstrFile=fn;
	ofn.nMaxFile=MAX_PATH;
	ofn.Flags=OFN_PATHMUSTEXIST|OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT;
	ofn.lpstrDefExt="png";

	if(!GetSaveFileName(&ofn)) return;  /* canceled by user */

	ZeroMemory((void*)&d2p,sizeof(PNGD_D2PINFO));

	d2p.structsize=sizeof(PNGD_D2PINFO);
	d2p.flags=0;
	d2p.pngfn=fn;
	d2p.lpdib=lpdib;
	d2p.dibsize=0;
	d2p.lpbits=0;
	d2p.bitssize=0;
	d2p.software="PNGDIB";
	d2p.errmsg = errmsg; strcpy(errmsg,"");

	ret=write_dib_to_png(&d2p);
	if(ret) {
		sprintf(buf,"Cannot save file: %s (%d)",errmsg,ret);
		MessageBox(hwnd,buf,MYNAME,MB_OK|MB_ICONWARNING);
	}
}

void init_globals(void)
{
	lpdib=NULL;
	use_bkgd=1; use_custom_bg=1; use_gamma=1;
	strcpy(png_filename,"");
	default_bg = RGB(255,255,255);
	image_bg=default_bg;
	custom_bg = RGB(240,240,240);
	which_bg=0;
	ZeroMemory((void*)g_custcolors,16*sizeof(COLORREF));
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
   LPSTR lpCmdLine,int nCmdShow)
{
	MSG msg;
	WNDCLASS  wc;
	HWND hwnd;

	init_globals();

	hInst=hInstance;
	hwnd=NULL;

	wc.style = CS_DBLCLKS;
	wc.lpfnWndProc = (WNDPROC)WndProcMain;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInst;
	wc.hIcon = LoadIcon(hInst,"ICONMAIN");
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = NULL;
	wc.lpszMenuName =  "MENUMAIN";
	wc.lpszClassName = "SMVIEWCLASS";
	if(!RegisterClass(&wc)) return 0;

	hwnd = CreateWindow(
		"SMVIEWCLASS",MYNAME,
		WS_VISIBLE|WS_OVERLAPPEDWINDOW|WS_CLIPCHILDREN,
		CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,
		NULL,NULL,hInst,NULL);
	if (!hwnd) return 0;

	DragAcceptFiles(hwnd,TRUE);

	while(GetMessage(&msg,NULL,0,0)){
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return (msg.wParam);
}

LRESULT CALLBACK WndProcMain(HWND hwnd, UINT msg,
     WPARAM wParam, LPARAM lParam)
{
	WORD id;
	id=LOWORD(wParam);

	switch(msg) {

	case WM_ERASEBKGND:
		{
			HDC hdc;
			RECT rect;
			HBRUSH oldbr,br;

			hdc=(HDC)wParam;
			if(!GetClientRect(hwnd,&rect)) return 0;
			if(which_bg) {
				br=CreateSolidBrush(image_bg);
			}
			else {
				if(use_custom_bg)
					br=CreateSolidBrush(custom_bg);
				else
					br=CreateSolidBrush(default_bg);
			}
			oldbr=(HBRUSH)SelectObject(hdc,br);
			FillRect(hdc,&rect,br);
			SelectObject(hdc,oldbr);
			DeleteObject(br);
		}
		return 1;

	case WM_PAINT:
		{
			PAINTSTRUCT paintStruct;
			HDC hdc;

			hdc = BeginPaint( hwnd, &paintStruct );
			if(lpdib) {
				StretchDIBits(hdc,
					0,0,lpdib->biWidth,lpdib->biHeight,
					0,0,lpdib->biWidth,lpdib->biHeight,
					lpbits,(LPBITMAPINFO)lpdib,
					DIB_RGB_COLORS,SRCCOPY);
			}
			EndPaint(hwnd, &paintStruct);
			return 0;
		}

	case WM_DESTROY:
		if(lpdib) { HeapFree(GetProcessHeap(),0,(LPVOID)lpdib); lpdib=NULL; }
		PostQuitMessage(0);
		return 0;

	case WM_DROPFILES:
		dropped_files(hwnd, (HDROP)wParam);
		return 0;

	case WM_INITMENU:
		CheckMenuItem((HMENU)wParam, ID_BG_BKGD , MF_BYCOMMAND|
			(use_bkgd?MF_CHECKED:MF_UNCHECKED) );
		CheckMenuItem((HMENU)wParam, ID_BG_CUSTOM , MF_BYCOMMAND|
			(use_custom_bg?MF_CHECKED:MF_UNCHECKED) );
		CheckMenuItem((HMENU)wParam, ID_GAMMACORRECT , MF_BYCOMMAND|
			(use_gamma?MF_CHECKED:MF_UNCHECKED) );
		EnableMenuItem((HMENU)wParam, ID_CLOSE, MF_BYCOMMAND|
			(lpdib?MF_ENABLED:MF_GRAYED) );
		EnableMenuItem((HMENU)wParam, ID_SAVEASPNG, MF_BYCOMMAND|
			(lpdib?MF_ENABLED:MF_GRAYED) );
		EnableMenuItem((HMENU)wParam, ID_SAVEASBMP, MF_BYCOMMAND|
			(lpdib?MF_ENABLED:MF_GRAYED) );
		return 0;

	case WM_COMMAND:
		switch(id) {
		case ID_EXIT:
			DestroyWindow(hwnd);
			return 0;
		case ID_OPEN:
			open_png_dialog(hwnd);
			return 0;
		case ID_SAVEASPNG:
			if(lpdib) save_as_png(hwnd);
			return 0;
		case ID_CLOSE:
			if(lpdib) { HeapFree(GetProcessHeap(),0,(LPVOID)lpdib); lpdib=NULL; }
			strcpy(png_filename,"");
			which_bg=0;
			InvalidateRect(hwnd,NULL,TRUE);
			return 0;
		case ID_BG_BKGD:
			use_bkgd = !use_bkgd;
			open_png_by_name(hwnd,png_filename); /* reload image */
			return 0;
		case ID_BG_CUSTOM:
			use_custom_bg = !use_custom_bg;
			open_png_by_name(hwnd,png_filename);
			return 0;
		case ID_GAMMACORRECT:
			use_gamma = !use_gamma;
			open_png_by_name(hwnd,png_filename);
			return 0;
		case ID_SETBG:
			if(choose_color_dialog(hwnd,&custom_bg)) {
				open_png_by_name(hwnd,png_filename);
			}
			return 0;
		case ID_ABOUT:
			DialogBox(hInst,"ABOUTDLG",hwnd,DlgProcAbout);
			return 0;
		}
		break;
	}
	return (DefWindowProc(hwnd, msg, wParam, lParam));
}

BOOL CALLBACK DlgProcAbout(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	WORD id;
	char buf[1000];
	id=LOWORD(wParam);

	switch (msg) {

	case WM_INITDIALOG: 
		sprintf(buf,"Demonstration PNG image viewer that uses the PNGDIB library.\r\n"
			"Version %s\r\nBy Jason Summers, 8/2000-4/2002\r\nPNGDIB version %s",
			SMVIEW_VERS,pngdib_get_version_string());
		SetDlgItemText(hwnd,IDC_TEXT1,buf);

		return 1;
	case WM_COMMAND:
		switch(id) {
		case IDOK:
		case IDCANCEL:
			EndDialog(hwnd, 0);
			return 1;
		}
	}
	return 0;
}
